/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.binding;

// Zeus imports
import org.enhydra.zeus.Binding;

/**
 * <p>
 *  <code>Property</code> implements the <code>{@link Binding}</code>
 *    interface and defines behavior for a property of a Java class.
 *    These properties have modifiers, types, and names, as well
 *    as a value.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public interface Property extends Binding {

    /** Access Modifier: <code>private</code> */
    public static final int ACCESS_PRIVATE = 0;

    /** Access Modifier: <code>private</code> */
    public static final int ACCESS_PROTECTED = 1;
    
    /** Access Modifier: <code>public</code> */        
    public static final int ACCESS_PUBLIC = 2;
    
    // XXX: At some point, there probably needs to be a way
    //      to set a property to static, volative, transient, etc.

    /**
     * <p>
     *  This will set the access level modifier for a property. The
     *    value submitted must be in the form of an <code>int</code>,
     *    which should correspond to one of the constants defined
     *    (<code>{@link #ACCESS_PRIVATE}</code>,
     *     <code>{@link #ACCESS_PROTECTED}</code>,
     *     or <code>{@link #ACCESS_PUBLIC}</code>. By default,
     *    all properties will be <code>private</code>
     *    (<code>ACCESS_PRIVATE</code>).
     * </p>
     *
     * @param modifier <code>int</code> constant for access level.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC          
     */
    public void setAccessModifier(int accessModifier);
    
    /**
     * <p>
     *  This will return the access level modifier for a property. The
     *    value returned will be in the form of an <code>int</code>,
     *    which will correspond to one of the constants defined
     *    (<code>{@link #ACCESS_PRIVATE}</code>,
     *     <code>{@link #ACCESS_PROTECTED}</code>,
     *     or <code>{@link #ACCESS_PUBLIC}</code>.
     * </p>
     *
     * @return <code>int</code> - constant for access level modifier.
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC          
     */    
    public int getAccessModifier();
    
    /**
     * <p>
     *  This will return the Java <code>String</code> representation
     *    of this <code>Property</code>'s access modifier. For
     *    example, <code>{@link #ACCESS_PRIVATE}</code> would be
     *    converted to "private".
     * </p>
     *
     * @return <code>String</code> - Java representation
     * @see #ACCESS_PRIVATE
     * @see #ACCESS_PROTECTED
     * @see #ACCESS_PUBLIC               
     */
    public String getAccessModifierString();
    
    /**
     * <p>
     *  This will set the default value of the property. Since no typing
     *    is available at this point, a simple Java <code>Object</code>
     *    is allowed as the type supplied. As a result, any errors in 
     *    mismatches between object type and allowed paramater type will
     *    occur at runtime, when class generation takes place. Supplying
     *    a value here essentially results in:
     * <code><pre>
     *    public class Foo {
     *
     *        private String myString = "some default value";
     *
     *        public String getMyString() {
     *            return myString;
     *        }
     *
     *        public void setMyString(String myString) {
     *            this.myString = myString;
     *        }
     * 
     *        // Other methods and properties
     *    }
     * </pre></code>
     * </p><p>
     *  Also, note that data binding users who supply their own class
     *    implementations will <b>LOSE THIS DEFAULT VALUE</b>, as the interface
     *    alone cannot specify a default value. So use this carefully!
     * </p>
     *
     * @param defaultValue <code>Object</code> to be used as default value.
     */
    public void setDefaultValue(Object defaultValue);
    
    /**
     * <p>
     *  This will retrieve the default value associated with this property,
     *    or <code>null</code> if there is not one. For more information on
     *    default property values, see <code>{@link #setDefaultValue(Object)}</code>.
     * </p>
     *
     * @return <code>Object</code> - default value of the property.
     */
    public Object getDefaultValue();
}