/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */
package org.enhydra.zeus.generator;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.List;

// Zeus imports
import org.enhydra.zeus.Binding;
import org.enhydra.zeus.Generator;
import org.enhydra.zeus.binding.AtomicProperty;
import org.enhydra.zeus.binding.Container;
import org.enhydra.zeus.binding.Property;
import org.enhydra.zeus.util.CapitalizationUtils;
import org.enhydra.zeus.util.ClassUtils;

/**
 * <p>
 *  <code>{@link Generator}</code> is the portion of Zeus that will convert
 *    a set of constraints, visible through a <code>{@link Binding}</code>,
 *    and output them as generated Java classes (to a 
 *    <code>File</code> [directory]).
 * </p><p>
 *  <code>SimpleGenerator</code> is the simplest form of a 
 *    <code>Generator</code> and takes in either a single <code>Binding</code>
 *    or a <code>List</code> of them, and directly generates Java classes.
 * </p>
 *
 * @author Brett McLaughlin
 * @version 1.0
 */
public class SimpleGenerator implements Generator {
    
    /**
     * <p>
     *  This will construct a new instance of <code>SimpleGenerator</code>.
     * </p>
     */
    public SimpleGenerator() {
    }

    /**
     * <p>
     *  This will convert from a set of constraints to Java classes.
     *    It accesses those constraints through the
     *    <code>{@link Binding}</code> interface, which provides
     *    them in a representation-independent format, and then
     *    converts them into Java code, which is output to the
     *    supplied <code>File</code>.
     * </p>
     *
     * @param binder <code>Binder</code> with ability to convert from
     *        constraints to Zeus <code>Binding</code>s.
     * @param directory <code>File</code> to output generated classes
     *        to.
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */
    public void generate(Binding binding, File directory) throws IOException {
        /** Ensure that we have a directory */
        if (!directory.isDirectory()) {
            throw new IOException("The File supplied was not a valid directory");
        }
    
        /**
         * Currently, only Containers can be generated (which
         *   includes ContainerProperties) as AtomicProperties
         *   won't result in classes.
         */
        if (binding instanceof Container) {
            Container container = (Container)binding;
        
            /** Generate interface */
            generateInterface(container, directory);
        
            /** Generate implementation class */
            generateImplementation(container, directory);            
        } else  {
            throw new IOException("Only containers can be used for class generation.");
        }
    }
    
    private void generateInterface(Container container, File directory) 
        throws IOException {
        
        // Get a new File handle
        String filename =
            new StringBuffer(container.getName())
                .append(".java")
                .toString();
        File outputFile = new File(directory, filename);
        FileWriter writer = new FileWriter(outputFile);
        
        writer.write("public interface ");
        writer.write(container.getName());
        writer.write(" {\n\n");
        
        // Write out each method
        List properties = container.getProperties();        
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            generatePropertyMethodDecl((Property)i.next(), writer);
        }        
        
        writer.write("}\n");    
        writer.flush();
    }
    
    private void generateImplementation(Container container, File directory) 
        throws IOException {
        
        // Get a new File handle
        String filename =
            new StringBuffer(container.getName())
                .append("Impl.java")
                .toString();
        File outputFile = new File(directory, filename);
        FileWriter writer = new FileWriter(outputFile);
                
        writer.write("public class ");
        writer.write(container.getName());
        writer.write("Impl implements ");
        writer.write(container.getName());
        writer.write(" {\n\n");
        
        // Write out each property
        List properties = container.getProperties();
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            generateProperty((Property)i.next(), writer);
        }
        
        writer.write("\n");
        
        // Write out each method implementation
        for (Iterator i = properties.iterator(); i.hasNext(); ) {
            generatePropertyMethodImpl((Property)i.next(), writer);
        }        
        
        writer.write("}\n");    
        writer.flush();
    }    
    
    /**
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */    
    private void generateProperty(Property property, Writer writer) 
        throws IOException {
        
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {
            AtomicProperty atomicProperty = (AtomicProperty)property;
            // Write out the property
            writer.write("    ");
            writer.write(atomicProperty.getAccessModifierString());
            writer.write(" ");
            writer.write(atomicProperty.getJavaType());
            writer.write(" ");
            writer.write(atomicProperty.getName());
            writer.write(";\n");
        }
    }
    
    /**
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */    
    private void generatePropertyMethodDecl(Property property, Writer writer) 
        throws IOException {
        
        // See if this is a Collection or not
        String javaType = property.getJavaType();
        boolean isCollection = ClassUtils.isCollectionClass(javaType);        
        
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {
            AtomicProperty atomicProperty = (AtomicProperty)property;
            
            // Write out the property accessor
            writer.write("    public ");
            writer.write(javaType);
            writer.write(" get");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getName()));
            if (isCollection) {
                writer.write("List");
            }            
            writer.write("();\n\n");
            
            // Write out the property mutator
            writer.write("    public void set");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getName()));
            if (isCollection) {
                writer.write("List");
            }            
            writer.write("(");
            writer.write(javaType);
            writer.write(" ");
            writer.write(atomicProperty.getName());
            writer.write(");\n\n");            
        }
    }    
    
    /**
     * @throws <code>IOException</code> - when errors in writing to the
     *         supplied <code>Result</code> occur.
     */    
    private void generatePropertyMethodImpl(Property property, Writer writer) 
        throws IOException {
        
        // See if this is a Collection or not
        String javaType = property.getJavaType();
        boolean isCollection = ClassUtils.isCollectionClass(javaType);
        
        /**
         * There is a conditional here, although all that ever should
         *   get to here are AtomicProperty objects. But just in case...
         */
        if (property instanceof AtomicProperty) {
            AtomicProperty atomicProperty = (AtomicProperty)property;
            
            // Write out the property accessor
            writer.write("    public ");
            writer.write(javaType);
            writer.write(" get");
            writer.write(
                CapitalizationUtils.initialUpper(atomicProperty.getName()));
            if (isCollection) {
                writer.write("List");
            }
            writer.write("() {\n");
            writer.write("        return ");
            writer.write(atomicProperty.getName());
            writer.write(";\n");    
            writer.write("    }\n\n");
            
            // Write out the property mutator
            writer.write("    public void set");
            writer.write(CapitalizationUtils.initialUpper(atomicProperty.getName()));
            if (isCollection) {
                writer.write("List");
            }            
            writer.write("(");
            writer.write(javaType);
            writer.write(" ");
            writer.write(atomicProperty.getName());
            writer.write(") {\n");
            writer.write("        this.");
            writer.write(atomicProperty.getName());
            writer.write(" = ");
            writer.write(atomicProperty.getName());
            writer.write(";\n");
            writer.write("    }\n\n");            
        }
    }    
}